package com.ggj.ggj12 
{
	import com.ggj.ggj12.geom.Line;
	import com.ggj.ggj12.input.InputManager;
	import com.ggj.ggj12.models.SnakeModel;
	import com.ggj.ggj12.utils.MathUtils;
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.utils.Timer;
	
	/**
	 * ...
	 * @author Matt Kucic
	 */
	public class CreationManager extends EventDispatcher 
	{
		private static const CREATE_NOT:int = 0;
		private static const CREATE_PADDING:int = 1;
		private static const CREATE_GROWING:int = 2;
		private static const CREATE_FINISHED:int = 3;
		
		private var _snake:SnakeModel;
		private var _creationState:int;
		private var _creationCounter:uint;
		private var _currentCreationID:uint;
		
		private var _inputManager:InputManager;
		private var _avergePnt:Point;
		
		private var _growingKey:int;
		private var _generatorSrcId:int;
		
		public function CreationManager(snakeData:SnakeModel, inputManager:InputManager, target:IEventDispatcher=null) 
		{
			super(target);
			_snake = snakeData;
			_currentCreationID = 0;
			_creationState = CREATE_NOT;

			_inputManager = inputManager;
		}
		
		public function update():void
		{
			switch(_creationState) {
				
				case CREATE_PADDING:
					updateCreatePadding();
					break;
				case CREATE_GROWING:
					updateCreateGrowing();
					break;
				case CREATE_FINISHED:
					updateCreateFinshed();
					break;
				case CREATE_NOT:
					updateCreateNot();
				default:
					break;
			}
		}

		private function headColliding():int
		{
			var head:Point = Point(_snake.segments[0]);
			for (var i:uint = 2; i < _snake.segments.length; i++) {
				var p:Point = Point(_snake.segments[i]);
				var dist:Point = p.subtract(head);
				if (dist.length < 5) { /* XXX magic number */
					return i;
				}
			}
			return 0;
			/*
			for (var i:uint = 2; i < _snake.segments.length - 1; i++)
			{
				if (MathUtils.lineLineIntersection(_snake.getLine(0),_snake.getLine(i)) != null)
				{
					return i;
				}
			}
			return 0;
			*/
		}
		
		private function updateCreateNot():void 
		{
			var collidingI:uint = headColliding();
			if (collidingI != 0) {
				SoundManager.getInstance().createSoundEffect(SoundManager.BITE, new Point(0, 0));
				
				_creationState = CREATE_PADDING;
				//get average point
				var creationX:Number=0;
				var creationY:Number=0;
				for (var j:uint = 0; j < collidingI; j++)
				{
					creationX += Point(_snake.segments[j]).x;
					creationY += Point(_snake.segments[j]).y;
				}
				_avergePnt = new Point((creationX / (collidingI)), (creationY / (collidingI)));
				
				var f:Function = function():Point { return _avergePnt; };
				_generatorSrcId = SoundManager.getInstance().createSoundSource(SoundManager.GENERATOR, f, .01, 0, true);
				_growingKey = _inputManager.activeKey;
				if (_growingKey == 0) {
					_growingKey = 1;
				}
				_creationCounter = 0;
			}		
		}
		
		private function updateCreateFinshed():void 
		{
			
		}
		
		private function updateCreateGrowing():void 
		{
			_creationCounter += 1;
			_snake.hasCreated = true;
			if (!headColliding()) {
				SoundManager.getInstance().removeSoundSource(_generatorSrcId);
				SoundManager.getInstance().createSoundEffect(SoundManager.BIRTH, new Point(0, 0));				
				dispatchEvent(new CreationEvent(CreationEvent.CREATE_END, _currentCreationID));
				_creationState = CREATE_NOT;
			}
		}
		
		private function updateCreatePadding():void 
		{
			_creationCounter += 1;
			var curVol:Number = SoundManager.getInstance().getSoundSource(_generatorSrcId).baseVol;
			curVol += .01;
			if (curVol <= 1) {
				SoundManager.getInstance().getSoundSource(_generatorSrcId).baseVol = curVol;
			}
			if (!headColliding()) {
				_creationState = CREATE_NOT;
				SoundManager.getInstance().removeSoundSource(_generatorSrcId);
			}
			else if (_creationCounter >= 30)
			{
				_currentCreationID += 1;
				dispatchEvent(new CreationEvent(CreationEvent.CREATE_START, _currentCreationID, _avergePnt));
				_creationState = CREATE_GROWING;
			}
		}
		
	}
}
