package com.ggj.ggj12 
{
	import com.ggj.ggj12.models.PlanetModel;
	import com.ggj.ggj12.view.PlanetView;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.utils.Timer;
	
	/**
	 * ...
	 * @author Matt Kucic
	 */
	public class Planet 
	{
		static public const STATE_INITIAL_GROWTH:uint = 0;
		static public const STATE_LEVEL0:uint = 1;
		static public const STATE_LEVEL1:uint = 2;
		static public const STATE_LEVEL2:uint = 3;
		static public const STATE_CONSUME:uint = 4;
		static public const STATE_EXCRETE:uint = 5;
		
		static public const RADIAL_INCREASE:Number = .1;
		static public const MAX_RADIUS:Number = 33;
		static public const TYPE_EARTH:uint = 0;
		static public const TYPE_MACHINE:uint = 1;
		static public const TYPE_FIRE:uint = 2;
		
		private var _model:PlanetModel;
		private var _view:PlanetView;
		private var _id:uint;
		private var _growing:Boolean;
		private var _currentState:uint;
		private var _alphaStep:int;
		private var _segCount:int;
		
		private var _baseAudioLoopId:int;
		private var _randSpawnFrame:Number;
		private var _maxEatables:Number;
		private var _eatablesSpawned:int;
		
		public function Planet(avergePnt:Point, id:uint) 
		{
			_id = id;
			_currentState = STATE_INITIAL_GROWTH;
			_model = new PlanetModel(avergePnt);
			_view = new PlanetView(_model);	
			trace("created ", id);
		}
		
		public function stopGrowth():void 
		{
			trace("Stop growth called for:" + _id);
			var typeLayer:Sprite = new Sprite();
			typeLayer.graphics.beginFill(Globals.groundColors[_model.type]);
			typeLayer.graphics.lineStyle(_model.radius / 4, Globals.crustColors[_model.type]);
			typeLayer.graphics.drawCircle(_model.radius+(_model.radius/8), _model.radius+(_model.radius/8), _model.radius);
			typeLayer.graphics.endFill();
			var typeLayerBitmapData:BitmapData = new BitmapData(typeLayer.width , typeLayer.height , true, 0x000000);
			typeLayerBitmapData.draw(typeLayer);
			var newBitmap:Bitmap = new Bitmap(typeLayerBitmapData,"auto",true);
			newBitmap.x = 0;
			newBitmap.y = 0;
			_view.addTypeLayer(newBitmap);
			_alphaStep = 0;
			_currentState = STATE_LEVEL0;
			
			var f:Function = function():Point { return _model.position; };			
			_baseAudioLoopId = SoundManager.getInstance().createSoundSource(audioIdOfTypeLevel(_model.type, STATE_LEVEL0), f);
		}
		
		public function update():void 
		{
			switch(_currentState)
			{
				case STATE_INITIAL_GROWTH:
					updateInitalGrowth();
					break;
				case STATE_LEVEL0:
					updateLevel0();
					break;
				case STATE_LEVEL1:
					updateLevel1();
					break;
				case STATE_LEVEL2:
					updateLevel2();
					break;
				case STATE_CONSUME:
					updateConsume();
					break;
				case STATE_EXCRETE:
					updateExcrete();
					break;
			}
			
		}
		
		private function updateExcrete():void 
		{
			//_model.setRadius(_model.radius * .9);
		}
		
		private function updateConsume():void 
		{
			SoundManager.getInstance().getSoundSource(_baseAudioLoopId).baseVol *= 0.9;
		}
		
		public function consume():void 
		{
			SoundManager.getInstance().createSoundEffect(SoundManager.DEVOUR, new Point(0, 0));
			_currentState = STATE_CONSUME;
			_view.consume();
		}
		
		public function startExcrete():void 
		{
			_currentState = STATE_EXCRETE;
			
		}
		
		public function destroy():void 
		{
			SoundManager.getInstance().removeSoundSource(_baseAudioLoopId);
		}
		
		private function updateInitalGrowth():void 
		{
			if (_model.radius < Planet.MAX_RADIUS)
			{
				_model.setRadius(_model.radius + Planet.RADIAL_INCREASE);
			}
			else {
				
			}
			_view.update();
		}
		
		private function audioIdOfTypeLevel(t:int, l:int):int
		{
			if (t == TYPE_EARTH) {
				if (l == STATE_LEVEL0) {
					return SoundManager.PLANT1;
				} else if (l == STATE_LEVEL1) {
					return SoundManager.PLANT12;
				} else if (l == STATE_LEVEL2) {
					return SoundManager.PLANT123;
				}
			} else if (t == TYPE_FIRE) {
				if (l == STATE_LEVEL0) {
					return SoundManager.FIRE1;
				} else if (l == STATE_LEVEL1) {
					return SoundManager.FIRE12;
				} else if (l == STATE_LEVEL2) {
					return SoundManager.FIRE123;
				}
				
			} else if (t == TYPE_MACHINE) {
				if (l == STATE_LEVEL0) {
					return SoundManager.METAL1;
				} else if (l == STATE_LEVEL1) {
					return SoundManager.METAL12;
				} else if (l == STATE_LEVEL2) {
					return SoundManager.METAL123;
				}								
			}
			return -1;
		}
		
		private function updateLevel0():void 
		{
			if (_view.typeLayer.alpha < 1) {
				if (_alphaStep % 5)
				{
					_view.typeLayer.alpha += .02;
				}
				_alphaStep += 1;
			}
			else {
				var baseSoundSrc:SoundSource = SoundManager.getInstance().getSoundSource(_baseAudioLoopId);
				baseSoundSrc.switchSound(SoundManager.getInstance().getSound(audioIdOfTypeLevel(_model.type, STATE_LEVEL1)));
				
				var center:Bitmap = new((Resources.artArry[_model.type])[0]) as Bitmap;
				_view.addCenter(center);
				
				_alphaStep = 0;
				_randSpawnFrame = Math.floor(Math.random() * ((_alphaStep + 150) - _alphaStep) + _alphaStep);
				_currentState = STATE_LEVEL1;
			}
			_view.update();
		}
		
		private function updateLevel1():void 
		{
			trace(_randSpawnFrame)
			if (_alphaStep == _randSpawnFrame) {
				var tuft:Bitmap = new((Resources.artArry[_model.type])[1]) as Bitmap;
				_view.addTuft(tuft);
				_randSpawnFrame = Math.floor(Math.random() * ((_alphaStep + 150) - _alphaStep) + _alphaStep);
			}
			_alphaStep ++;
			_view.update1();
			
			if (_alphaStep >= 600)
			{
				var baseSoundSrc:SoundSource = SoundManager.getInstance().getSoundSource(_baseAudioLoopId);
				baseSoundSrc.switchSound(SoundManager.getInstance().getSound(audioIdOfTypeLevel(_model.type, STATE_LEVEL2)));

				_alphaStep = 0;
				_maxEatables = Math.ceil(_model.radius / 5);
				_eatablesSpawned = 0;
				_randSpawnFrame = Math.floor(Math.random() * ((_alphaStep + 300) - _alphaStep) + _alphaStep);
				_currentState = STATE_LEVEL2;
			}
			
		}
		
		private function updateLevel2():void 
		{
			trace("step 2: " + _randSpawnFrame +" "+_alphaStep);
			if (_alphaStep == _randSpawnFrame && _maxEatables > _eatablesSpawned) {
				var eatable:Bitmap = new((Resources.artArry[_model.type])[2]) as Bitmap;
				_view.addEatable(eatable);
				_eatablesSpawned ++;
				_randSpawnFrame = Math.floor(Math.random() * ((_alphaStep + 300) - _alphaStep) + _alphaStep);
			}
			_alphaStep ++;
			_view.update2();
			
			if (_maxEatables <= _eatablesSpawned)
			{
				trace("done spawning")
			}
		}
		
		public function get view():PlanetView 
		{
			return _view;
		}
		
		public function get id():uint 
		{
			return _id;
		}
		
		public function get currentState():uint 
		{
			return _currentState;
		}
		
		public function get model():PlanetModel 
		{
			return _model;
		}
		
		public function get segCount():int 
		{
			return _segCount;
		}
		
		public function set segCount(value:int):void 
		{
			_segCount = value;
		}
		
	}

}