package com.ggj.ggj12 
{
	import com.ggj.ggj12.input.InputManager;
	import com.ggj.ggj12.models.SnakeModel;
	import com.ggj.ggj12.utils.MathUtils;
	import com.ggj.ggj12.view.SnakeView;
	import com.ggj.ggj12.view.UniverseView;
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.Dictionary;
	
	
	/**
	 * ...
	 * @author Matt Kucic
	 */
	public class UniverseController extends EventDispatcher 
	{
		private var _universeView:UniverseView;
		private var _snakeModel:SnakeModel;
		private var _snakeView:SnakeView;
		private var _creationManager:CreationManager;
		private var _planets:Dictionary;
		private var _indicators:Dictionary;
		//private var _snakeController:SnakeController;
		
		private var _frameCount:Number;
		
		public function UniverseController(inputManager:InputManager) 
		{
			_universeView = new UniverseView();
			_planets = new Dictionary();
			_indicators = new Dictionary();
			_snakeModel = new SnakeModel(64, new Point(300,300));
			_snakeView = new SnakeView(_snakeModel);
			_universeView.addSnake(_snakeView);
			_frameCount = 0;
			
			_creationManager = new CreationManager(_snakeModel, inputManager);
			_creationManager.addEventListener(CreationEvent.CREATE_START, onCreationStartEvent);
			_creationManager.addEventListener(CreationEvent.CREATE_END, onCreationEndEvent);
			
			var soundManager:SoundManager = SoundManager.getInstance();			
			var originPos:Function = function():Point { return new Point(300, 300); };
			var snakePos:Function = function():Point { return Point(_snakeModel.segments[0]); };
			soundManager.createSoundSource(SoundManager.BACKGROUND, snakePos, 0.2, 0, true);
		}
		
		private function onCreationStartEvent(e:CreationEvent):void 
		{
			var newPlanet:Planet = new Planet(e.avergePnt, e.currentCreationID);
			var getSnakePos:Function = function():Point { return Point(_snakeModel.segments[0]); };
			
			var indicator:Indicator = new Indicator(getSnakePos, e.avergePnt, 800, 800);
			_universeView.addChild(indicator);
			_indicators[e.currentCreationID] = indicator;
			indicator.draw();
			
			_universeView.addPlanet(newPlanet.view);
			_planets[e.currentCreationID] = newPlanet;
		}
		private function onCreationEndEvent(e:CreationEvent):void 
		{
			trace("Creation end event: " + e + " " + e.currentCreationID);
			if (_planets[e.currentCreationID] != null)
			{
				Planet(_planets[e.currentCreationID]).stopGrowth();
			}
			
		}
		
		
		public function advance(velocity:Number,theta:Number,delta:Number = 0):void
		{
			_frameCount++;
			//delta is not used yet
			
			_snakeModel.reposition(velocity, theta);
			if (theta > 0 || _snakeModel.hasCreated == true)
			{
				_snakeView.hideArrows();
			}
			else {
				_snakeView.showArrows();
			}
			_snakeView.update();
			for each(var planet:Planet in _planets)
			{
				planet.update();
				//check for collision
				if (planet.currentState == Planet.STATE_INITIAL_GROWTH) {
					if (_frameCount % 30 == 0) {
						planet.segCount++;
						_snakeModel.segments.pop();
					}
				} else if (planet.currentState >= Planet.STATE_LEVEL1)
				{	
					//trace("Model point: " + planet.model.position + " head point: " + Point(_snakeModel.segments[0]));
					//trace("Distance: " + MathUtils.distance(planet.model.position, Point(_snakeModel.segments[0])) + " Radius: " + planet.model.radius);
					if ((MathUtils.distance(planet.model.position, Point(_snakeModel.segments[0]))) < planet.model.radius)
					{
						planet.consume();
					}
				}
				if (planet.currentState == Planet.STATE_CONSUME)
				{
					//trace("Check for Excrete");
					if ((MathUtils.distance(planet.model.position, Point(_snakeModel.segments[_snakeModel.segments.length-1]))) < planet.model.radius)
					{
						planet.startExcrete();
					}
				}
				else if (planet.currentState == Planet.STATE_EXCRETE)
				{
					//trace("Check for death");
					if ((MathUtils.distance(planet.model.position, Point(_snakeModel.segments[_snakeModel.segments.length-1]))) > planet.model.radius)
					{
						_snakeModel.addSegments(planet.segCount);
						
						_universeView.removeChild(Indicator(_indicators[planet.id]));
						delete _indicators[planet.id];

						delete _planets[planet.id];
						_universeView.removePlanet(planet.view);
						planet.destroy();
						planet = null;
						
					}
				}
			}
			_creationManager.update();
			_universeView.scrollRect = new Rectangle(Point(_snakeModel.segments[0]).x - 400, Point(_snakeModel.segments[0]).y - 400, 800, 800);
			//_universeView.updateSnake(_snakeView)
			
			for each(var indicator:Indicator in _indicators)
			{
				indicator.update();
			}
			
			SoundManager.getInstance().adjustAll(Point(_snakeModel.segments[0]));
		}
		
		public function getUniverseView():UniverseView 
		{
			return _universeView;
		}
		
		public function get snakeModel():SnakeModel 
		{
			return _snakeModel;
		}
		
	}

}