package com.ggj.ggj12.utils 
{
	import com.ggj.ggj12.geom.Line;
	import flash.geom.Point;
	/**
	 * ...
	 * @author Matt Kucic
	 */
	public class MathUtils 
	{
		
		/**
		 * 
		 * Retrieved from: http://keith-hair.net/blog/2008/08/04/find-intersection-point-of-two-lines-in-as3/
		 * 
		 * Checks for intersection of Segment if as_seg is true.
		 * Checks for intersection of Line if as_seg is false.
		 * Return intersection of Segment AB and Segment EF as a Point
		 * Return null if there is no intersection
		 */ 		
		public static function lineIntersectLine(A:Point,B:Point,E:Point,F:Point,as_seg:Boolean=true):Point {
			var ip:Point;
			var a1:Number;
			var a2:Number;
			var b1:Number;
			var b2:Number;
			var c1:Number;
			var c2:Number;
		 
			a1= B.y-A.y;
			b1= A.x-B.x;
			c1= B.x*A.y - A.x*B.y;
			a2= F.y-E.y;
			b2= E.x-F.x;
			c2= F.x*E.y - E.x*F.y;
		 
			var denom:Number=a1*b2 - a2*b1;
			if (denom == 0) {
				return null;
			}
			ip=new Point();
			ip.x=(b1*c2 - b2*c1)/denom;
			ip.y=(a2*c1 - a1*c2)/denom;
		 
			//---------------------------------------------------
			//Do checks to see if intersection to endpoints
			//distance is longer than actual Segments.
			//Return null if it is with any.
			//---------------------------------------------------
			if(as_seg){
				if(Math.pow(ip.x - B.x, 2) + Math.pow(ip.y - B.y, 2) > Math.pow(A.x - B.x, 2) + Math.pow(A.y - B.y, 2))
				{
				   return null;
				}
				if(Math.pow(ip.x - A.x, 2) + Math.pow(ip.y - A.y, 2) > Math.pow(A.x - B.x, 2) + Math.pow(A.y - B.y, 2))
				{
				   return null;
				}
		 
				if(Math.pow(ip.x - F.x, 2) + Math.pow(ip.y - F.y, 2) > Math.pow(E.x - F.x, 2) + Math.pow(E.y - F.y, 2))
				{
				   return null;
				}
				if(Math.pow(ip.x - E.x, 2) + Math.pow(ip.y - E.y, 2) > Math.pow(E.x - F.x, 2) + Math.pow(E.y - F.y, 2))
				{
				   return null;
				}
			}
			return ip;
		} 
		
		public static function lineLineIntersection(line1:Line,line2:Line):Point
		{
			// Denominator for ua and ub are the same, so store this calculation
			var d:Number =
				(line2.pointB.y - line2.pointA.y) * (line1.pointB.x - line1.pointA.x)
				-
				(line2.pointB.x - line2.pointA.x) * (line1.pointB.y - line1.pointA.y);

			//n_a and n_b are calculated as seperate values for readability
			var n_a:Number =
				(line2.pointB.x - line2.pointA.x) * (line1.pointA.y - line2.pointA.y)
				-
				(line2.pointB.y - line2.pointA.y) * (line1.pointA.x - line2.pointA.x);

			var n_b:Number =
				(line1.pointB.x - line1.pointA.x) * (line1.pointA.y - line2.pointA.y)
				-
				(line1.pointB.y - line1.pointA.y) * (line1.pointA.x - line2.pointA.x);
		
			// Make sure there is not a division by zero - this also indicates that
			// the lines are parallel.  
			// If n_a and n_b were both equal to zero the lines would be on top of each 
			// other (coincidental).  This check is not done because it is not 
			// necessary for this implementation (the parallel check accounts for this).
			if (d == 0){
				return null;
			}

			// Calculate the intermediate fractional point that the lines potentially intersect.
			var ua:Number = n_a / d;
			var ub:Number = n_b / d;

			// The fractional point will be between 0 and 1 inclusive if the lines
			// intersect.  If the fractional calculation is larger than 1 or smaller
			// than 0 the lines would need to be longer to intersect.
			
			if (ua >= 0 && ua <= 1 && ub >= 0 && ub <= 1)
			{
				var interPt:Point = new Point();
				interPt.x = line1.pointA.x + (ua * (line1.pointB.x - line1.pointA.x));
				interPt.y = line1.pointA.y + (ua * (line1.pointB.y - line1.pointA.y));
				return interPt;
			}
			return null;
		}
		
		public static function distance(p1:Point, p2:Point):Number
		{
			return Math.sqrt(Math.pow((p2.x-p1.x),2)+Math.pow((p2.y-p1.y),2));
		}
		
	}

}